;**************************************************************************
;* FileName	: BASE.ASM
;* Date 	: 10.02.2003
;* File version : v1.00
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.atw.hu
;* E-mail	: majki.mester@gmail.com
;*
;*-------------------------------------------------------------------------
;*
;* Description	: Test for BASE
;*		  ==================================
;*		  
;*
;*
;*-------------------------------------------------------------------------
;*
;* Connections	:	       _______	 _______
;*			      |       \_/	|
;*		     -reset ==| -MCLR	    RB7 |== n.c.
;*		       n.c. ==| RA0	    RB6 |== n.c.
;*		       n.c. ==| RA1	    RB5 |== n.c.
;*		       n.c. ==| RA2	    RB4 |== n.c.
;*		       n.c. ==| RA3	    RB3 |== n.c.
;*		       n.c. ==| RA4	    RB2 |== n.c.
;*		       n.c. ==| RA5	    RB1 |== n.c.
;*		       n.c. ==| RE0	    RB0 |== n.c.
;*		       n.c. ==| RE1	    Vdd |== +5V
;*		       n.c. ==| RE2	    Vss |== GND
;*		        GND ==| Vss	    RD7 |== n.c.
;*		        +5V ==| Vdd	    RD6 |== n.c.
;*		    quartz1 ==| OSC1	    RD5 |== n.c.
;*		    quartz2 ==| OSC2	    RD4 |== n.c.
;*		       n.c. ==| RC0	    RC7 |== n.c.
;*		       n.c. ==| RC1	    RC6 |== n.c.
;*		       n.c. ==| RC2	    RC5 |== n.c.
;*		       n.c. ==| RC3	    RC4 |== n.c.
;*		       n.c. ==| RD0	    RD3 |== n.c.
;*		       n.c. ==| RD1	    RD2 |== n.c.
;*			      |_________________|
;*				  PIC16F877A 
;*
;*-------------------------------------------------------------------------
;*
;* Resource	: Ports 	  :    xx /   33 pin   (   %)
;*		  Data memory	  :    xx /  368 byte  (   %)
;*		  Prog memory	  :   xxx / 8192 word  (   %)
;*		  EEPROM	  :    xx /  256 byte  (   %)
;*		  Timers	  : 
;*		  Interrupt	  : 
;*		  CCP		  :
;*		  MSSP		  :
;*		  USART		  :
;*		  AD		  :
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  10/02/2003 	Initial Release
;*
;**************************************************************************

	list	p=16f877a, r=dec, l=0

;***** INCLUDES ***********************************************************

	include "p16f877a.inc"
	include "..\..\modules\base.inc"

;***** CONFIG ************************************************************

CFG_1	EQU 	_XT_OSC & _WDT_OFF & _PWRTE_ON & _BODEN_ON & _LVP_OFF
CFG_2	EQU	_CPD_OFF & _WRT_OFF & _DEBUG_OFF & _CP_OFF

	__CONFIG CFG_1 & CFG_2
	__IDLOCS 0100h			; version

	ERRORLEVEL -306			; supress message (Crossing page boundary)
	ERRORLEVEL -307			; supress message (Setting page bits.)
	MULTIPAGE 4, 2048		; support for multipage program memory
	EXPAND_MACROS	EXP_COND	; macro expansion mode

;***** DEFINES ************************************************************
;#define DEBUG				; for some debug message
;#define NO_DELAY			; for fast simulation
;#define NO_RS232			; for fast simulation
;#define NO_EE24

xtal_freq	EQU	4 MHz	  	; crystal frequency
core_freq	EQU	xtal_freq/4	; core operating frequency


; ----- porta -------------------------------------------------------------

#define		PORTA_0		PORTA,0		; free
#define		PORTA_1		PORTA,1		; free
#define		PORTA_2		PORTA,2		; free
#define		PORTA_3		PORTA,3		; free
#define		PORTA_4		PORTA,4		; free (open drain!)
#define		PORTA_5		PORTA,5		; free

; ----- portb -------------------------------------------------------------

#define		PORTB_0		PORTB,0		; free
#define		PORTB_1		PORTB,1		; free
#define		PORTB_2		PORTB,2		; free
#define		PORTB_3		PORTB,3		; free
#define		PORTB_4		PORTB,4		; free
#define		PORTB_5		PORTB,5		; free
#define		PORTB_6		PORTB,6		; free
#define		PORTB_7		PORTB,7		; free

; ----- portc -------------------------------------------------------------

#define		PORTC_0		PORTC,0		; free
#define		PORTC_1		PORTC,1		; free
#define		PORTC_2		PORTC,2		; free
#define		PORTC_3		PORTC,3		; free
#define		PORTC_4		PORTC,4		; free
#define		PORTC_5		PORTC,5		; free
#define		PORTC_6		PORTC,6		; free
#define		PORTC_7		PORTC,7		; free

; ----- portd -------------------------------------------------------------

#define		PORTD_0		PORTD,0		; free
#define		PORTD_1		PORTD,1		; free
#define		PORTD_2		PORTD,2		; free
#define		PORTD_3		PORTD,3		; free
#define		PORTD_4		PORTD,4		; free
#define		PORTD_5		PORTD,5		; free
#define		PORTD_6		PORTD,6		; free
#define		PORTD_7		PORTD,7		; free

; ----- porte -------------------------------------------------------------

#define		PORTE_0		PORTE,0		; free
#define		PORTE_1		PORTE,1		; free
#define		PORTE_2		PORTE,2		; free


;***** MACROS *************************************************************

	include "..\..\modules\dummy.mac"

;***** CONSTANT DEFINITIONS ***********************************************

option_val	EQU	10000010b	; Timer source : intclk / 8
trisa_val	EQU	00111111b	; TRISA value (all input)
trisb_val	EQU	11111111b	; TRISB value (all input)
trisc_val	EQU	11111111b	; TRISC value (all input)
trisd_val	EQU	11111111b	; TRISD value (all input)
trise_val	EQU	00000111b	; TRISE value (all input)
adcon0_val	EQU	00000000b	; ADCON0 value (AD off)
adcon1_val	EQU	00000110b	; ADCON1 value (add digital input)
intcon_val	EQU	00100000b	; Interrupt control

buffer_length	EQU	16
;--------------------------------------------------------------------------

#define		FLAG_FIRST	flags,0
#define		FLAG_SECOND	flags,1


;***** VARIABLE DEFINITIONS ***********************************************
	cblock	70h			; common RAM
		icd_temp		; reserved for ICD
		w_temp			; variable used for context saving
		status_temp		; variable used for context saving
		pclath_temp		; variable used for context saving
		fsr_temp		; variable used for context saving
	endc

;--------------------------------------------------------------------------
	cblock	1EBh
		icd_temp2:5		; reserved for ICD
	endc

;--------------------------------------------------------------------------
	cblock	20h

		TEMP			; dummy register
		X, Y

		buffer:buffer_length	
		flags


	endc



;##########################################################################
;# PAGE 0
;##########################################################################

;***** ENTRY POINT ********************************************************
		ORG	000h		; processor reset vector
		nop
		clrwdt
		clrf	STATUS
		goto	start


;***** IT HANDLER *********************************************************
;* Interrupt handler
;* <comment>
;**************************************************************************
		ORG	004h		; interrupt vector location
		movwf	w_temp		; save off current W register contents
		movf	STATUS,w	; move STATUS register into W register
		movwf	status_temp	; save off contents of STATUS register
		movf	PCLATH,w	; move PCLATH register into w register
		movwf	pclath_temp	; save off contents of PCLATH register

		movf	FSR,w		; save FSR
		movwf	fsr_temp
;--------------------------------------------------------------------------

		; <IT code>

;--------------------------------------------------------------------------
		movf	fsr_temp,w	; restore FSR
		movwf	FSR

		movf	pclath_temp,w	; retrieve copy of PCLATH register
		movwf	PCLATH		; restore pre-isr PCLATH register contents
		movf	status_temp,w	; retrieve copy of STATUS register
		movwf	STATUS		; restore pre-isr STATUS register contents
		swapf	w_temp,f
		swapf	w_temp,w	; restore pre-isr W register contents
		retfie			; return from interrupt




;***** REAL PROGRAM START *************************************************
start
;----- set configuration --------------------------------------------------
		clrf	PORTA		; clear latch
		clrf	PORTB		; clear latch
		clrf	PORTC		; clear latch
		clrf	PORTD		; clear latch
		clrf	PORTE		; clear latch
		bank1
		ERRORLEVEL -302
		movlw	option_val
		movwf	OPTION_REG	; set OPTION_REG
		movlw	trisa_val
		movwf	TRISA		; set TRISA
		movlw	trisb_val
		movwf	TRISB		; set TRISB
		movlw	trisc_val
		movwf	TRISC		; set TRISC
		movlw	trisd_val
		movwf	TRISD		; set TRISD
		movlw	trise_val
		movwf	TRISE		; set TRISE
		movlw	adcon1_val	
		movwf	ADCON1		; set ADCON1
		bank0
		ERRORLEVEL +302
		movlw	adcon0_val	
		movwf	ADCON0		; set ADCON0
		movlw	intcon_val	
		movwf	INTCON		; set INTCON

		IT_ENABLE

;----- main cycle  --------------------------------------------------------

		clrf 	flags

		movlw	100
		movwf	X
		movlw	200
		movwf	Y
		XCHG	X, Y

		movlw	50
		XCHGW	X


		movlw	10
		movwf	buffer
		movlw	10
		movwf	buffer+buffer_length-1


		MEM_CLEAR buffer, buffer_length

		IT_DISABLE


	
;----- main cycle  --------------------------------------------------------

main_cycle	nop

		fcall	sub_0
		fcall	sub_1
		fcall	sub_2
		fcall	sub_3

		nop



		GOTO_IF_1 FLAG_FIRST, label1

		nop
		nop

label1		GOTO_IF_0 FLAG_SECOND, label2

		nop
		nop

label2


		fgoto	main_cycle






;***** sub_0 **************************************************************
;* 
;* Input :              		; 
;* Output:  				; 
;* Clock :   
;**************************************************************************
sub_0		nop
		return



;***** PAGE 0 INCLUDES ****************************************************

	include "..\..\modules\dummy.inc"

;**************************************************************************



;##########################################################################
;# PAGE 1
;##########################################################################
		ORG	0800h


;***** sub_1 **************************************************************
;* 
;* Input :              		; 
;* Output:  				; 
;* Clock :   
;**************************************************************************
sub_1		nop
		return



;***** PAGE 1 INCLUDES ****************************************************

	; include "..\..\modules\dummy.inc"

;**************************************************************************







;##########################################################################
;# PAGE 2
;##########################################################################
		ORG	1000h


;***** sub_2 **************************************************************
;* 
;* Input :              		; 
;* Output:  				; 
;* Clock :   
;**************************************************************************
sub_2		nop
		return



;***** PAGE 2 INCLUDES ****************************************************

	; include "..\..\modules\dummy.inc"

;**************************************************************************







;##########################################################################
;# PAGE 3
;##########################################################################
		ORG	1800h

;***** sub_3 **************************************************************
;* 
;* Input :              		; 
;* Output:  				; 
;* Clock :   
;**************************************************************************
sub_3		nop
		return



;***** PAGE 3 INCLUDES ****************************************************

	; include "..\..\modules\dummy.inc"

;**************************************************************************
	end				  ; end of program

